<?php

/**
 * @file
 * The class for PoolParty Taxonomy Manager.
 * Export / Import / Sync methods.
 */

class PPTaxonomyManager {
  protected $config;

  /**
   * Constructor of the PPTaxonomyManager class.
   *
   * @param object $config
   *   A PoolParty Taxonomy Manager configuration.
   */
  public function __construct($config) {
    $this->config = $config;
  }

  /**
   * Returns a list of all additional fields for a PoolParty taxonomy.
   *
   * @return array
   *   A list additional fields.
   */
  protected static function taxonomyFields() {
    $taxonomy_field_schema = array(
      'field_uri' => (object) array(
        'name' => 'field_uri',
        'type' => 'text',
        'label' => t('URI'),
        'description' => t('The URI of the concept'),
        'required' => TRUE,
        'settings' => array(
          'max_length' => 255,
        ),
        'widget' => array(
          'type' => 'text_textfield',
          'weight' => 3,
        ),
      ),
      'field_alt_labels' => (object) array(
        'name' => 'field_alt_labels',
        'type' => 'text',
        'label' => t('Alternative labels'),
        'description' => t('A comma separated list of synonyms'),
        'required' => FALSE,
        'settings' => array(
          'max_length' => 8191,
        ),
        'widget' => array(
          'type' => 'text_textfield',
          'weight' => 4,
        ),
        'property' => 'skos:altLabel',
      ),
      'field_hidden_labels' => (object) array(
        'name' => 'field_hidden_labels',
        'type' => 'text',
        'label' => t('Hidden labels'),
        'description' => t('A comma separated list of secondary name variants of this term'),
        'required' => FALSE,
        'settings' => array(
          'max_length' => 8191,
        ),
        'widget' => array(
          'type' => 'text_textfield',
          'weight' => 5,
        ),
        'property' => 'skos:hiddenLabel',
      ),
    );

    // Add the possibility to add custom fields  via hook here.
    $custom_fields = array();
    drupal_alter('pp_taxonomy_manager_custom_attributes', $custom_fields);
    if (!empty($custom_fields)) {
      foreach ($custom_fields as $field_id => $custom_field) {
        // Check if a property is given and it is not one of the custom_fields
        if (isset($custom_field->property) && !isset($taxonomy_field_schema[$field_id]) && isset($custom_field->type) && $custom_field->type == 'text') {
          $taxonomy_field_schema[$field_id] = $custom_field;
        }
      }
    }

    return $taxonomy_field_schema;
  }

  /**
   * Returns the PoolParty Taxonomy Manager configuration.
   *
   * @return object
   *   A PoolParty Taxonomy Manager configuration object.
   */
  public function getConfig() {
    return $this->config;
  }

  /**
   * Set the correct translation mode for the Drupal taxonomy.
   *
   * @param object $taxonomy
   *   A Drupal taxonomy.
   * @param array $languages
   *   An array of languages:
   *    key = Drupal language
   *    value = PoolParty language.
   */
  public function setTranslationMode($taxonomy, $languages) {
    if (!module_exists('i18n_taxonomy')) {
      return;
    }

    $language_count = count($languages);
    $i18n_mode_old = $taxonomy->i18n_mode;

    // Make the taxonomy translatable if the translation module for taxonomies
    // is installed and more than one language is selected or the translation
    // mode is wrong.
    if (($language_count > 1 && $taxonomy->i18n_mode != I18N_MODE_TRANSLATE) || !in_array($taxonomy->i18n_mode, array(
        I18N_MODE_NONE,
        I18N_MODE_TRANSLATE,
      ))
    ) {
      $taxonomy->i18n_mode = I18N_MODE_TRANSLATE;
      taxonomy_vocabulary_save($taxonomy);
    }

    // If the translation mode was wrong, then change it for all terms.
    if ($language_count > 1 && $i18n_mode_old == I18N_MODE_LOCALIZE) {
      $this->changeTranslationMode($taxonomy, $languages);
    }
  }

  /**
   * Changes the translation mode of a taxonomy from "Localize" to "Translate".
   *
   * @param object $taxonomy
   *   A Drupal taxonomy object.
   * @param array $languages
   *   An array of languages:
   *    key = Drupal language
   *    value = PoolParty language.
   */
  protected function changeTranslationMode($taxonomy, $languages) {
    $default_language = language_default('language');
    $selected_languages = array_keys($languages);

    $processed_terms = array();
    $parents = array();

    // Go through all taxonomy terms.
    $tree = taxonomy_get_tree($taxonomy->vid, 0, NULL, TRUE);
    foreach ($tree as $term) {
      if (isset($processed_terms[$term->tid])) {
        continue;
      }
      $processed_terms[$term->tid] = TRUE;
      $translation_set = NULL;
      $parents[$term->tid] = array();

      // Go through all selected languages.
      foreach ($selected_languages as $language) {
        $translate_term = clone $term;
        $save_translation = FALSE;

        // Create a new translation set if the language is the default language.
        if ($language == $default_language) {
          $save_translation = TRUE;
          $translation_set = i18n_translation_set_create('taxonomy_term', $taxonomy->machine_name);
        }

        // For all other languages get its translations if exists and create new
        // term object.
        else {
          $translations = i18n_string_translation_search('taxonomy:term:' . $term->tid . ':*', $language);
          if (!empty($translations)) {
            $save_translation = TRUE;
            $translate_term->tid = NULL;
            $translate_term->parent = NULL;
            foreach ($translations as $translation) {
              $translate_term->{$translation->property} = $translation->translations[$language];
            }
          }
        }

        // Save the term and add it to the translation set.
        if ($translation_set && $save_translation) {
          // Add the parents for the terms with non-default language.
          if ($language != $default_language) {
            foreach ($term->parents as $parent) {
              if (!empty($parents[$parent][$language])) {
                $translate_term->parent[] = $parents[$parent][$language];
              }
            }
          }
          // Save the term.
          $translate_term->language = $language;
          taxonomy_term_save($translate_term);
          // Add the term to the translation set.
          $translation_set->add_item($translate_term, $language);
          $translation_set->save();

          if ($language != $default_language) {
            $parents[$term->tid][$language] = $translate_term->tid;
          }
        }
      }
    }
  }

  /**
   * Creates a new Drupal taxonomy.
   *
   * @param object $root_object
   *   Either a PoolParty concept scheme or a PoolParty project.
   * @param string $taxonomy_name
   *   The name of the taxonomy to create If NULL is given the title of the
   *   concept scheme gets used (default).
   *
   * @return object
   *   The created taxonomy.
   */
  public function createTaxonomy($root_object, $taxonomy_name = '') {
    $taxonomy_name = trim(check_plain($taxonomy_name));
    if (empty($taxonomy_name)) {
      $taxonomy_name = $root_object->title;
    }

    // Check if the new taxonomy already exists.
    $machine_name = self::machineName($taxonomy_name);
    $taxonomy = taxonomy_vocabulary_machine_name_load($machine_name);

    if ($taxonomy === FALSE) {
      // Set the data for the new taxonomy.
      $taxonomy = (object) array(
        'name' => $taxonomy_name,
        'machine_name' => $machine_name,
        'description' => substr(t('Automatically created by PoolParty Taxonomy Manager.') . ' ' . (isset($root_object->description) ? $root_object->description : (isset($root_object->descriptions) && !empty($root_object->descriptions) ? $root_object->descriptions[0] : '')), 0, 128),
      );

      // Create the taxonomy.
      taxonomy_vocabulary_save($taxonomy);

      drupal_set_message(t('Vocabulary %taxonomy successfully created.', array('%taxonomy' => $taxonomy_name)));
      watchdog('PoolParty Taxonomy Manager', 'Vocabulary created: %taxonomy (VID = %vid)', array(
        '%taxonomy' => $taxonomy_name,
        '%vid' => $taxonomy->vid,
      ));
    }

    return $taxonomy;
  }

  /**
   * Adds additional fields to a specific taxonomy term if not exists.
   *
   * @param object $taxonomy
   *   A Drupal taxonomy object.
   */
  public function adaptTaxonomyFields($taxonomy) {
    $fields = self::taxonomyFields();
    foreach ($fields as $field) {
      $this->createTaxonomyTermField($field);
      $this->addFieldToTaxonomyTerm($field, $taxonomy);
    }
  }

  /**
   * Creates a new field if not exists.
   *
   * @param object $field
   *   The field that should be created.
   */
  protected function createTaxonomyTermField($field) {
    $exists = field_info_field($field->name) ? TRUE : FALSE;
    if (!$exists) {
      $new_field = array(
        'field_name' => $field->name,
        'type' => $field->type,
        'label' => $field->label,
        'settings' => $field->settings,
      );
      field_create_field($new_field);
    }
  }

  /**
   * Adds a field to a specific taxonomy term if not exists.
   *
   * @param object $field
   *   The field that should be added.
   * @param object $taxonomy
   *   The taxonomy at which the field should be added.
   */
  protected function addFieldToTaxonomyTerm($field, $taxonomy) {
    $fields = field_info_instances('taxonomy_term', $taxonomy->machine_name);
    if (!isset($fields[$field->name])) {
      $instance = array(
        'field_name' => $field->name,
        'entity_type' => 'taxonomy_term',
        'bundle' => $taxonomy->machine_name,
        'label' => $field->label,
        'description' => $field->description,
        'required' => $field->required,
        'widget' => $field->widget,
      );
      field_create_instance($instance);
    }
  }

  /**
   * Creates a new concept scheme on the specified PoolParty server.
   *
   * @param object $taxonomy
   *   A Drupal taxonomy object.
   * @param string $scheme_title
   *   The title of the concept scheme to create. If NULL is given the name of
   *   the taxonomy gets used (default).
   *
   * @return string
   *   The URI of the new concept scheme.
   */
  public function createConceptScheme($taxonomy, $scheme_title = '') {
    $scheme_title = trim(check_plain($scheme_title));
    if (empty($scheme_title)) {
      $scheme_title = $taxonomy->name;
    }

    $description = 'Automatically created by Drupal. ' . $taxonomy->description;
    /* @var $ppt SemanticConnectorPPTApi */
    $ppt = $this->config->getConnection()->getAPI('PPT');
    $scheme_uri = $ppt->createConceptScheme($this->config->getProjectId(), $scheme_title, $description);
    drupal_set_message(t('Concept scheme %scheme successfully created.', array('%scheme' => $scheme_title)));
    watchdog('PoolParty Taxonomy Manager', 'Concept scheme created: %scheme (URI = %uri)', array(
      '%scheme' => $scheme_title,
      '%uri' => $scheme_uri,
    ));

    return $scheme_uri;
  }

  /**
   * Creates a new project on the specified PoolParty server.
   *
   * @param object $taxonomy
   *   A Drupal taxonomy object.
   * @param string $project_title
   *   The title of the project to create. If NULL is given the name of
   *   the taxonomy gets used (default).
   * @param string $default_language
   *   The language code of the language to use as default for the new project
   *   in PoolParty.
   * @param array $available_languages
   *   A complete array of languages available for the new PoolParty project;
   *   If this array is empty only the default language will be used.
   *
   * @return string
   *   The ID of the new project.
   */
  public function createProject($taxonomy, $project_title = '', $default_language = 'en', array $available_languages = array()) {
    $project_title = trim(check_plain($project_title));
    if (empty($project_title)) {
      $project_title = $taxonomy->name;
    }

    if (empty($available_languages)) {
      $available_languages = array($default_language);
    }

    $description = 'Automatically created by Drupal. ' . $taxonomy->description;
    /* @var $ppt SemanticConnectorPPTApi */
    $ppt = $this->config->getConnection()->getAPI('PPT');

    $project_id = $ppt->createProject($project_title, $default_language, array('Public'), array('description' => $description, 'availableLanguages' => $available_languages));
    drupal_set_message(t('Project %project successfully created.', array('%project' => $project_title)));
    watchdog('PoolParty Taxonomy Manager', 'Project created: %project (ID = %id)', array(
      '%project' => $project_title,
      '%id' => $project_id,
    ));

    return $project_id;
  }

  /**
   * Creates a batch for exporting all terms of a taxonomy.
   *
   * @param object $taxonomy
   *   A Drupal taxonomy object.
   * @param string $root_uri
   *   A concept scheme URI or project ID.
   * @param array $languages
   *   An array of languages:
   *    key = Drupal language
   *    value = PoolParty language.
   * @param int $terms_per_request
   *   Count of taxonomy terms per http request.
   */
  public function exportTaxonomyTerms($taxonomy, $root_uri, $languages, $terms_per_request) {
    $start_time = time();

    // Configure the batch data.
    $batch = array(
      'title' => t('Exporting taxonomy %name ...', array('%name' => $taxonomy->name)),
      'operations' => array(),
      'init_message' => t('Starting with the export of the taxonomy terms.'),
      'progress_message' => t('Processed @current out of @total.'),
      'finished' => 'pp_taxonomy_manager_taxonomy_export_finished',
    );

    // Get the taxonomy tree for the default language.
    $default_language = language_default('language');
    if (module_exists('i18n_taxonomy') && $taxonomy->i18n_mode == I18N_MODE_TRANSLATE) {
      $tree = i18n_taxonomy_get_tree($taxonomy->vid, $default_language, 0, NULL, TRUE);
    }
    else {
      $tree = taxonomy_get_tree($taxonomy->vid, 0, NULL, TRUE);
    }

    // Set additional data.
    $count = count($tree);
    $info = array(
      'total' => $count,
      'start_time' => $start_time,
    );

    // Set the export operations.
    for ($i = 0; $i < $count; $i += $terms_per_request) {
      $terms = array_slice($tree, $i, $terms_per_request);
      $batch['operations'][] = array(
        'pp_taxonomy_manager_taxonomy_export',
        array(
          $this,
          $terms,
          $default_language,
          $languages[$default_language],
          $root_uri,
          $info,
        ),
      );
    }

    // Set the update hash table operations after the export of all terms.
    for ($i = 0; $i < $count; $i += $terms_per_request) {
      $terms = array_slice($tree, $i, $terms_per_request);
      $batch['operations'][] = array(
        'pp_taxonomy_manager_taxonomy_update_hash',
        array($this, $terms, $info),
      );
    }

    // Set the export translation operations.
    unset($languages[$default_language]);
    if (!empty($languages)) {
      foreach ($languages as $drupal_lang => $pp_lang) {
        $tree = i18n_taxonomy_get_tree($taxonomy->vid, $drupal_lang, 0, NULL, TRUE);
        $count = count($tree);
        $info = array(
          'total' => $count,
          'start_time' => $start_time,
        );

        for ($i = 0; $i < $count; $i += $terms_per_request) {
          $terms = array_slice($tree, $i, $terms_per_request);
          $batch['operations'][] = array(
            'pp_taxonomy_manager_taxonomy_export_translations',
            array($this, $terms, $drupal_lang, $pp_lang, $info),
          );
        }
      }
    }

    // Set the log operation.
    $batch['operations'][] = array(
      'pp_taxonomy_manager_taxonomy_log_save',
      array($this, $taxonomy->vid, $info),
    );

    // Start the batch.
    batch_set($batch);
  }

  /**
   * Batch process method for exporting taxonomy terms into a PoolParty server.
   *
   * @param array $terms
   *   The taxonomy terms that are to be exported.
   * @param string $drupal_lang
   *   The language of the taxonomy terms that are to be exported.
   * @param string $pp_lang
   *   The language of the concept that are to be created.
   * @param array $context
   *   The batch context to transmit data between different calls.
   */
  public function exportBatch(array $terms, $drupal_lang, $pp_lang, array &$context) {
    $config = $this->getConfig();
    /* @var $ppt SemanticConnectorPPTApi */
    $ppt = $config->getConnection()->getAPI('PPT');

    // Custom attribute fields.
    $custom_fields = self::taxonomyFields();
    $normal_fields = array(
      'field_uri',
      'field_alt_labels',
      'field_hidden_labels',
    );
    foreach ($normal_fields as $normal_field) {
      unset($custom_fields[$normal_field]);
    }

    $settings = $this->config->getConfig();
    $exported_terms = &$context['results']['exported_terms'];
    $project_id = ($settings['root_level'] == 'conceptscheme' ? $config->getProjectId() : $exported_terms[0]['uri']);
    foreach ($terms as $term) {
      // Create new concept.
      $created = FALSE;
      if (!isset($exported_terms[$term->tid])) {
        foreach ($term->parents as $parent) {
          if (isset($exported_terms[$parent])) {
            // Create new concept.
            if (!$created) {
              if ($settings['root_level'] == 'project' && $parent == 0) {
                $uri = $ppt->createConceptScheme($project_id, $term->name, (!empty($term->description) ? $term->description : ''));
              }
              else {
                $uri = $ppt->createConcept($project_id, $term->name, $exported_terms[$parent]['uri']);
                // Add definition, alt labels, hidden labels and custom properties
                // if required.
                if (!empty($term->description)) {
                  $ppt->addLiteral($project_id, $uri, 'definition', $term->description, $pp_lang);
                }
                if (!empty($term->field_alt_labels)) {
                  $alt_labels = explode(',', $term->field_alt_labels[LANGUAGE_NONE][0]['value']);
                  foreach ($alt_labels as $alt_label) {
                    $ppt->addLiteral($project_id, $uri, 'alternativeLabel', $alt_label, $pp_lang);
                  }
                }
                if (!empty($term->field_hidden_labels)) {
                  $hidden_labels = explode(',', $term->field_hidden_labels[LANGUAGE_NONE][0]['value']);
                  foreach ($hidden_labels as $hidden_label) {
                    $ppt->addLiteral($project_id, $uri, 'hiddenLabel', $hidden_label, $pp_lang);
                  }
                }
                if (!empty($custom_fields)) {
                  foreach ($custom_fields as $field_id => $field_schema) {
                    if (!empty($term->{$field_id})) {
                      $ppt->addCustomAttribute($project_id, $uri, $field_schema->property, $term->{$field_id}[LANGUAGE_NONE][0]['value'], $pp_lang);
                    }
                  }
                }
              }

              // Update term with the new URI.
              $term->field_uri[LANGUAGE_NONE][0]['value'] = $uri;
              taxonomy_term_save($term);
              $exported_terms[$term->tid] = array(
                'uri' => $uri,
                'parents' => array($parent),
                'drupalLang' => $drupal_lang,
                'ppLang' => $pp_lang,
                'hash' => FALSE,
              );
              watchdog('PoolParty Taxonomy Manager', 'Concept created: %name (URI = %uri)', array(
                '%name' => $term->name,
                '%uri' => $uri,
              ));
              $created = TRUE;
            }
            // Add additional parents.
            else {
              $relation_type = ($settings['root_level'] != 'project' || $parent != 0) ? 'broader' : 'topConceptOf';
              $uri = $exported_terms[$term->tid]['uri'];
              $ppt->addRelation($project_id, $uri, $exported_terms[$parent]['uri'], $relation_type);
              $exported_terms[$term->tid]['parents'][] = $parent;
            }
          }
        }
      }
      // Add missing parents to the concept.
      else {
        foreach ($term->parents as $parent) {
          if (isset($exported_terms[$parent]) && !in_array($parent, $exported_terms[$term->tid]['parents'])) {
            $relation_type = ($settings['root_level'] != 'project' || $parent != 0) ? 'broader' : 'topConceptOf';
            $uri = $exported_terms[$term->tid]['uri'];
            $ppt->addRelation($project_id, $uri, $exported_terms[$parent]['uri'], $relation_type);
            $exported_terms[$term->tid]['parents'][] = $parent;
          }
        }
      }
      $context['results']['processed']++;
    }
  }

  /**
   * Batch process function for updating the hash table after the export.
   *
   * @param array $terms
   *   The taxonomy terms that are to be exported.
   * @param array $info
   *   An associative array of information about the batch process.
   * @param array $context
   *   The batch context to transmit data between different calls.
   */
  public function updateHashBatch(array $terms, array $info, array &$context) {
    $config = $this->getConfig();
    /* @var $ppt SemanticConnectorPPTApi */
    $ppt = $config->getConnection()->getAPI('PPT');

    $settings = $config->getConfig();
    $exported_terms = &$context['results']['exported_terms'];
    $project_id = ($settings['root_level'] == 'conceptscheme' ? $config->getProjectId() : $exported_terms[0]['uri']);
    foreach ($terms as $term) {
      if (isset($exported_terms[$term->tid]) && !$exported_terms[$term->tid]['hash']) {
        // Add hash data to the database.
        $data = $exported_terms[$term->tid];
        // The term is a concept scheme.
        if ($settings['root_level'] == 'project' && $term->parents == array(0)) {
          $concept = new stdClass();
          $schemes = $ppt->getConceptSchemes($project_id, $data['ppLang']);
          foreach ($schemes as $scheme) {
            if ($scheme->uri == $data['uri']) {
              $concept = $scheme;
            }
          }
        }
        // The term is a concept.
        else {
          $concept = $ppt->getConcept($project_id, $data['uri'], $this->skosProperties(), $data['ppLang']);
        }
        $concept->drupalLang = $data['drupalLang'];
        $concept->ppLang = $data['ppLang'];

        $uri_lang = $this->getUri($concept);
        $hash = $this->hash($concept);
        $this->addHashData($term, $uri_lang, $hash, $info['start_time']);

        $exported_terms[$term->tid]['hash'] = TRUE;
        $context['results']['hash_update_processed']++;
      }
    }
  }

  /**
   * Batch process method for exporting taxonomy terms into a PoolParty server.
   *
   * @param array $terms
   *   The taxonomy terms that are to be exported.
   * @param string $drupal_lang
   *   The language of the taxonomy terms that are to be exported.
   * @param string $pp_lang
   *   The language of the concept that are to be created.
   * @param array $info
   *   An associative array of information about the batch process.
   * @param array $context
   *   The batch context to transmit data between different calls.
   */
  public function exportTranslationsBatch(array $terms, $drupal_lang, $pp_lang, array $info, array &$context) {
    $config = $this->getConfig();
    /* @var $ppt SemanticConnectorPPTApi */
    $ppt = $config->getConnection()->getAPI('PPT');

    // Custom attribute fields.
    $custom_fields = self::taxonomyFields();
    $normal_fields = array(
      'field_uri',
      'field_alt_labels',
      'field_hidden_labels',
    );
    foreach ($normal_fields as $normal_field) {
      unset($custom_fields[$normal_field]);
    }

    $default_language = language_default('language');
    $settings = $config->getConfig();
    $exported_terms = &$context['results']['exported_terms'];
    $project_id = ($settings['root_level'] == 'conceptscheme' ? $config->getProjectId() : $exported_terms[0]['uri']);
    foreach ($terms as $term) {
      // Get the taxonomy term ID from the default language.
      $default_tid = i18n_taxonomy_translation_term_tid($term->tid, $default_language, FALSE);

      // Check if the term with the default language is already exported.
      if ($default_tid && isset($exported_terms[$default_tid])) {
        $uri = $exported_terms[$default_tid]['uri'];
        // Add pref, alt, hidden labels and definition.
        $ppt->addLiteral($project_id, $uri, 'preferredLabel', $term->name, $pp_lang);
        if (!empty($term->description)) {
          $ppt->addLiteral($project_id, $uri, 'definition', $term->description, $pp_lang);
        }
        if ($settings['root_level'] == 'conceptscheme' || $term->parents != array(0)) {
          if (!empty($term->field_alt_labels)) {
            $alt_labels = explode(',', $term->field_alt_labels[LANGUAGE_NONE][0]['value']);
            foreach ($alt_labels as $alt_label) {
              $ppt->addLiteral($project_id, $uri, 'alternativeLabel', $alt_label, $pp_lang);
            }
          }
          if (!empty($term->field_hidden_labels)) {
            $hidden_labels = explode(',', $term->field_hidden_labels[LANGUAGE_NONE][0]['value']);
            foreach ($hidden_labels as $hidden_label) {
              $ppt->addLiteral($project_id, $uri, 'hiddenLabel', $hidden_label, $pp_lang);
            }
          }
          // Add custom properties if required.
          if (!empty($custom_fields)) {
            foreach ($custom_fields as $field_id => $field_schema) {
              if (!empty($term->{$field_id})) {
                $ppt->addCustomAttribute($project_id, $uri, $field_schema->property, $term->{$field_id}[LANGUAGE_NONE][0]['value'], $pp_lang);
              }
            }
          }
        }

        // Update term with the new URI.
        $term->field_uri[LANGUAGE_NONE][0]['value'] = $uri;
        taxonomy_term_save($term);

        // Add hash data to the database.
        // The term is a concept scheme.
        if ($settings['root_level'] == 'project' && $term->parents == array(0)) {
          $concept = new stdClass();
          $schemes = $ppt->getConceptSchemes($project_id, $pp_lang);
          foreach ($schemes as $scheme) {
            if ($scheme->uri == $uri) {
              $concept = $scheme;
            }
          }
        }
        // The term is a concept.
        else {
          $concept = $ppt->getConcept($project_id, $uri, $this->skosProperties(), $pp_lang);
        }
        $concept->drupalLang = $drupal_lang;
        $concept->ppLang = $pp_lang;

        $uri_lang = $this->getUri($concept);
        $hash = $this->hash($concept);
        $this->addHashData($term, $uri_lang, $hash, $info['start_time']);
      }
    }
    $context['results']['translation_processed']++;
  }

  /**
   * Creates a batch for updating all terms of a taxonomy.
   *
   * @param object $taxonomy
   *   A Drupal taxonomy object.
   * @param string $root_uri
   *   The URI of a concept scheme or the ID of a project
   * @param array $languages
   *   An array of languages:
   *    key = Drupal language
   *    value = PoolParty language.
   * @param int $concepts_per_request
   *   Count of concepts per http request.
   */
  public function updateTaxonomyTerms($taxonomy, $root_uri, $languages, $concepts_per_request) {
    $start_time = time();

    // Configure the batch data.
    $batch = array(
      'title' => t('Updating taxonomy %name ...', array('%name' => $taxonomy->name)),
      'operations' => array(),
      'init_message' => t('Starting with the update of the taxonomy terms.'),
      'progress_message' => t('Processed @current out of @total.'),
      'finished' => 'pp_taxonomy_manager_taxonomy_update_finished',
    );

    // Get all the concepts from the PoolParty server per language.
    $skos_properties = $this->skosProperties();
    /* @var $ppt SemanticConnectorPPTApi */
    $ppt = $this->config->getConnection()->getApi('PPT');

    $settings = $this->config->getConfig();
    $top_term_uris = array();
    $concepts = array();
    if ($settings['root_level'] == 'project') {
      foreach ($languages as $drupal_lang => $pp_lang) {
        $concept_schemes = $ppt->getConceptSchemes($root_uri, $pp_lang);
        foreach ($concept_schemes as $concept_scheme) {
          // Get the "hasTopConcept" relations of the scheme.
          $top_concepts = $ppt->getTopConcepts($root_uri, $concept_scheme->uri, $skos_properties, $pp_lang);
          $concept_scheme_children = array();
          if (is_array($top_concepts)) {
            foreach ($top_concepts as $top_concept) {
              $concept_scheme_children[] = $top_concept->uri;
            }
          }

          // Add the concept scheme itself.
          $concept_scheme_object = (object) array(
            'uri' => $concept_scheme->uri,
            'prefLabel' => $concept_scheme->title,
            'narrowers' => $concept_scheme_children,
            'drupalLang' => $drupal_lang,
            'ppLang' => $pp_lang,
          );
          $concepts[$this->getUri($concept_scheme_object)] = $concept_scheme_object;

          $top_term_uris[] = $concept_scheme->uri . '@' . $pp_lang;
          // Add the top concepts and concepts.
          $tree = $ppt->getSubTree($root_uri, $concept_scheme->uri, $skos_properties, $pp_lang);
          $tree_list = $this->tree2list($tree, $drupal_lang, $pp_lang);

          // Add the fake broader relation to the concept scheme for all top
          // concepts.
          if (is_array($tree)) {
            foreach ($tree_list as &$concept) {
              if (in_array($concept->uri, $concept_scheme_children)) {
                $concept->broaders = array($concept_scheme->uri);
              }
              unset($top_concept);
            }
          }
          $concepts = array_merge($concepts, $tree_list);
        }
      }
    }
    else {
      foreach ($languages as $drupal_lang => $pp_lang) {
        $top_concepts = $ppt->getTopConcepts($this->config->getProjectId(), $root_uri, $skos_properties, $pp_lang);
        foreach ($top_concepts as $top_concept) {
          $top_term_uris[] = $top_concept->uri . '@' . $pp_lang;
        }
        // Includes top concepts and concepts.
        $tree = $ppt->getSubTree($this->config->getProjectId(), $root_uri, $skos_properties, $pp_lang);
        $tree_list = $this->tree2list($tree, $drupal_lang, $pp_lang);
        $concepts = array_merge($concepts, $tree_list);
      }
    }

    // Set additional data.
    $count = count($concepts);
    $info = array(
      'total' => $count,
      'start_time' => $start_time,
      'top_concept_uris' => $top_term_uris,
    );

    // Set the update operations.
    for ($i = 0; $i < $count; $i += $concepts_per_request) {
      $concept_list = array_slice($concepts, $i, $concepts_per_request);
      $batch['operations'][] = array(
        'pp_taxonomy_manager_taxonomy_update',
        array(
          $this,
          $concept_list,
          $taxonomy->vid,
          $taxonomy->machine_name,
          $info,
        ),
      );
    }

    // Set the update parents operations.
    for ($i = 0; $i < $count; $i += $concepts_per_request) {
      $concept_list = array_slice($concepts, $i, $concepts_per_request);
      $batch['operations'][] = array(
        'pp_taxonomy_manager_taxonomy_update_parents',
        array($this, $concept_list, $info),
      );
    }

    // Set the delete operations for the removed concepts.
    $batch['operations'][] = array(
      'pp_taxonomy_manager_taxonomy_delete',
      array($this, $taxonomy->vid),
    );

    // Set the log operation.
    $batch['operations'][] = array(
      'pp_taxonomy_manager_taxonomy_log_save',
      array($this, $taxonomy->vid, $info),
    );

    // Start the batch.
    batch_set($batch);
  }

  /**
   * Batch process method for updating taxonomy terms from a PoolParty server.
   *
   * @param array $concepts
   *   The concepts that are to be updated.
   * @param string $vid
   *   The taxonomy ID where the terms should be updated.
   * @param string $machine_name
   *   The taxonomy machine_name where the terms should be updated.
   * @param array $info
   *   An associative array of information about the batch process.
   * @param array $context
   *   The batch context to transmit data between different calls.
   */
  public function updateBatch($concepts, $vid, $machine_name, $info, &$context) {
    $uris = array_keys($concepts);
    $processed_uris = array();

    // Get mapping data for every concept.
    $result = db_select('pp_taxonomy_manager_terms', 't')
      ->fields('t', array('tid', 'uri', 'hash'))
      ->condition('t.tmid', $this->config->getId())
      ->condition('t.vid', $vid)
      ->condition('t.uri', $uris)
      ->execute();

    // Update all existing terms.
    while ($record = $result->fetch()) {
      $concept = $concepts[$record->uri];
      $hash = $this->hash($concept);
      if ($record->hash != $hash) {
        $term = taxonomy_term_load($record->tid);
        $term = $this->mapTaxonomyTermDetails($term, $concept);
        taxonomy_term_save($term);
        $this->updateHashData($term, $hash, $info['start_time']);
        $context['results']['updated_terms'][$record->uri] = $record->tid;
        watchdog('PoolParty Taxonomy Manager', 'Taxonomy term updated: %name (TID = %tid)', array(
          '%name' => $term->name,
          '%tid' => $term->tid,
        ));
      }
      else {
        $context['results']['skipped_terms'][$record->uri] = $record->tid;
      }

      $processed_uris[] = $record->uri;
      $context['results']['processed']++;
    }

    // Create new terms for new existing concepts.
    $new_uris = array_diff($uris, $processed_uris);
    if (!empty($new_uris)) {
      foreach ($new_uris as $uri) {
        // Check if a term with the same URI and language already exists
        // in the taxonomy.
        $concept = $concepts[$uri];
        $query = db_select('field_data_field_uri', 'u')
          ->fields('u', array('entity_id'))
          ->condition('u.bundle', $machine_name)
          ->condition('u.field_uri_value', $concept->uri);
        $query->join('taxonomy_term_data', 't', 'u.entity_id = t.tid');

        // Add language condition if multilingual for taxonomies is enabled.
        if (module_exists('i18n_taxonomy')) {
          $query->condition('t.language', $concept->drupalLang);
        }

        // Get the taxonomy term ID if the URI exists.
        $tid = $query->execute()
          ->fetchField();

        $term_exists = FALSE;
        if ($tid) {
          $term = taxonomy_term_load($tid);
          if ($term !== FALSE) {
            $term_exists = TRUE;
          }
        }

        if (!$term_exists) {
          $term = new stdClass();
          $term->vid = $vid;
        }

        // Map data and save the term.
        $term = $this->mapTaxonomyTermDetails($term, $concept);
        taxonomy_term_save($term);

        // Add the term to a translation set.
        if (module_exists('i18n_taxonomy')) {
          $default_language = language_default('language');
          if ($term->language == $default_language) {
            $translation_set = ($term->i18n_tsid) ? i18n_translation_set_load($term->i18n_tsid) : i18n_translation_set_create('taxonomy_term', $machine_name);
          }
          else {
            // Get the translation set ID from the default term.
            $query = db_select('taxonomy_term_data', 't')
              ->fields('t', array('i18n_tsid'))
              ->condition('t.language', $default_language)
              ->condition('u.bundle', $machine_name)
              ->condition('u.field_uri_value', $concept->uri);
            $query->join('field_data_field_uri', 'u', 't.tid = u.entity_id');
            $i18n_tsid = $query->execute()
              ->fetchField();
            $translation_set = ($i18n_tsid) ? i18n_translation_set_load($i18n_tsid) : i18n_translation_set_create('taxonomy_term', $machine_name);
          }
          $translation_set->add_item($term, $term->language);
          $translation_set->save();
        }

        // Add the hash to the hash table.
        $uri = $this->getUri($concept);
        $hash = $this->hash($concept);
        $this->addHashData($term, $uri, $hash, $info['start_time']);
        $context['results']['created_terms'][$uri] = $term->tid;
        $context['results']['processed']++;
        watchdog('PoolParty Taxonomy Manager', 'Taxonomy term created: %name (TID = %tid)', array(
          '%name' => $term->name,
          '%tid' => $term->tid,
        ));
      }
    }
  }

  /**
   * Batch process method for updating the parents for the taxonomy terms.
   *
   * @param array $concepts
   *   The concepts that are to be updated.
   * @param array $info
   *   An associative array of information about the batch process.
   * @param array $context
   *   The batch context to transmit data between different calls.
   */
  public function updateParentsBatch($concepts, $info, &$context) {
    $changed_terms = $context['results']['updated_terms'] + $context['results']['created_terms'];
    $all_terms = $changed_terms + $context['results']['skipped_terms'];
    foreach ($concepts as $concept) {
      $concept_uri = $this->getUri($concept);

      // Check if concept is updated or new.
      if (!isset($changed_terms[$concept_uri])) {
        $context['results']['processed_parents']++;
        continue;
      }

      // If the concept is a top concept then set to the top of the tree.
      $parents = array();
      if (in_array($concept_uri, $info['top_concept_uris'])) {
        $parents[] = 0;
      }
      if (isset($concept->broaders) && !empty($concept->broaders)) {
        foreach ($concept->broaders as $broader) {
          $broader_uri = $broader . '@' . $concept->ppLang;
          if (isset($all_terms[$broader_uri])) {
            $parents[] = $all_terms[$broader_uri];
          }
        }
      }
      $term = taxonomy_term_load($all_terms[$concept_uri]);
      $term->parent = empty($parents) ? 0 : $parents;
      taxonomy_term_save($term);

      $context['results']['processed_parents']++;
    }
  }

  /**
   * Batch process method for deleting taxonomy terms.
   *
   * @param string $vid
   *   The taxonomy ID where the terms should be updated.
   * @param array $context
   *   The batch context to transmit data between different calls.
   */
  public function deleteBatch($vid, &$context) {
    $all_terms = $context['results']['updated_terms'] + $context['results']['created_terms'] + $context['results']['skipped_terms'];
    $all_terms = array_values($all_terms);
    $result_query = db_select('pp_taxonomy_manager_terms', 't')
      ->fields('t', array('tid', 'uri'))
      ->condition('t.tmid', $this->config->getId())
      ->condition('t.vid', $vid);

    if (!empty($all_terms)) {
      $result_query->condition('t.tid', $all_terms, 'NOT IN');
    }
    $result = $result_query->execute();

    while ($record = $result->fetch()) {
      $term = taxonomy_term_load($record->tid);
      taxonomy_term_delete($record->tid);
      $context['results']['deleted_terms'][$record->uri] = $record->tid;
      watchdog('PoolParty Taxonomy Manager', 'Taxonomy term deleted: %name (TID = %tid)', array(
        '%name' => $term->name,
        '%tid' => $term->tid,
      ));
    }

    // Delete the "Concepts" term if it was created by the PowerTagging module.
    $query = db_select('taxonomy_term_data', 't')
      ->fields('t', array('tid'))
      ->condition('t.vid', $vid)
      ->condition('t.name', 'Concepts');
    $query->leftJoin('field_data_field_uri', 'u', 't.tid = u.entity_id AND u.entity_type = \'taxonomy_term\'');
    $query->isNull('u.field_uri_value');
    $concepts_tid = $query->execute()->fetchField();
    if (!empty($concepts_tid)) {
      taxonomy_term_delete($concepts_tid);
    }
  }

  /**
   * Deletes the term from the hash table.
   *
   * @param object $term
   *   The taxonomy term.
   */
  public static function deleteTaxonomyTerm($term) {
    db_delete('pp_taxonomy_manager_terms')
      ->condition('vid', $term->vid)
      ->condition('tid', $term->tid)
      ->execute();
  }

  /**
   * Inserts the new statistic log.
   *
   * @param int $vid
   *   The Drupal taxonomy ID.
   * @param int $start_time
   *   The start time of the batch.
   * @param int $end_time
   *   The end time of the batch.
   */
  public function addLog($vid, $start_time, $end_time) {
    global $user;

    db_insert('pp_taxonomy_manager_logs')
      ->fields(array(
        'tmid' => $this->config->getId(),
        'vid' => $vid,
        'start_time' => $start_time,
        'end_time' => $end_time,
        'uid' => $user->uid,
      ))
      ->execute();
  }

  /**
   * Deletes all synchronization data.
   *
   * @param int $vid
   *   The Drupal taxonomy ID.
   */
  public function deleteSyncData($vid) {
    // Delete the log data.
    db_delete('pp_taxonomy_manager_logs')
      ->condition('tmid', $this->config->getId())
      ->condition('vid', $vid)
      ->execute();

    // Delete the hash data.
    db_delete('pp_taxonomy_manager_terms')
      ->condition('tmid', $this->config->getId())
      ->condition('vid', $vid)
      ->execute();
  }

  /**
   * Calculates the remaining time of a batch process.
   *
   * @param int $start_time
   *   The start time.
   * @param int $processed
   *   The count of processed items.
   * @param int $total
   *   The total count of items.
   *
   * @return string
   *   The remaining time in a human readable string.
   */
  public function calculateRemainingTime($start_time, $processed, $total) {
    $time_string = '';
    if ($processed > 0) {
      $remaining_time = floor((time() - $start_time) / $processed * ($total - $processed));
      if ($remaining_time > 0) {
        $time_string = (floor($remaining_time / 3600) % 24) . ' hours ' . (floor($remaining_time / 60) % 60) . ' minutes ' . ($remaining_time % 60) . ' seconds';
      }
      else {
        $time_string = t('Done.');
      }
    }

    return $time_string;
  }

  /**
   * Maps a taxonomy term data with a PoolParty concept.
   *
   * @param object $term
   *   The object of the taxonomy term, which will receive the new detail data.
   * @param object $concept
   *   A concept detail data to update the term with.
   *
   * @return object
   *   The mapped taxonomy term.
   */
  protected function mapTaxonomyTermDetails($term, $concept) {
    $term->name = $concept->prefLabel;
    $term->language = $concept->drupalLang;
    $term->field_uri[LANGUAGE_NONE][0]['value'] = $concept->uri;
    if (isset($concept->definitions)) {
      $term->description = implode(' ', $concept->definitions);
    }
    if (isset($concept->altLabels)) {
      // Remove multibyte-characters.
      $term->field_alt_labels[LANGUAGE_NONE][0]['value'] = preg_replace('/[[:^print:]]/', "", implode(',', $concept->altLabels));
    }
    if (isset($concept->hiddenLabels)) {
      // Remove multibyte-characters.
      $term->field_hidden_labels[LANGUAGE_NONE][0]['value'] = preg_replace('/[[:^print:]]/', "", implode(',', $concept->hiddenLabels));
    }

    // Add data for custom fields.
    if (isset($concept->properties)) {
      $fields = self::taxonomyFields();
      foreach ($fields as $field_id => $field_schema) {
        if (!in_array($field_id, array(
            'field_uri',
            'field_alt_labels',
            'field_hidden_labels',
          )) && isset($concept->properties->{$field_schema->property})
        ) {
          $term->{$field_id}[LANGUAGE_NONE][0]['value'] = $concept->properties->{$field_schema->property};
        }
      }
    }

    return $term;
  }

  /**
   * Creates a list of concepts from a tree.
   *
   * @param array $tree
   *   A list of concepts in tree format.
   * @param string $drupal_lang
   *   The Drupal language of the concepts.
   * @param string $pp_lang
   *   The PoolParty project language of the concepts.
   * @param int $depth
   *   The depth of the recursive function call.
   *
   * @return array
   *   A list of concept objects.
   */
  protected function tree2list($tree, $drupal_lang, $pp_lang, $depth = 0) {
    $concepts = array();
    foreach ($tree as $subtree) {
      if (is_object($subtree) && !empty($subtree->concept)) {
        // If a concept is in the top level but is not a top concept, then
        // remove its broaders (it's important for other languages if is not
        // translated consistently).
        if ($depth == 0) {
          unset($subtree->concept->broaders);
        }
        $subtree->concept->drupalLang = $drupal_lang;
        $subtree->concept->ppLang = $pp_lang;
        $concept_uri = $this->getUri($subtree->concept);
        $concepts[$concept_uri] = $subtree->concept;
        if (!empty($subtree->narrowers)) {
          $tree_list = $this->tree2list($subtree->narrowers, $drupal_lang, $pp_lang, ($depth + 1));
          $concepts = array_merge($concepts, $tree_list);
        }
      }
    }
    return $concepts;
  }

  /**
   * Inserts the new created concept to the database.
   *
   * @param object $term
   *   A Drupal taxonomy term.
   * @param string $uri
   *   The URI with language prefix of a concept.
   * @param string $hash
   *   The new hash data.
   * @param int $start_time
   *   The start time of the batch.
   */
  protected function addHashData($term, $uri, $hash, $start_time) {
    db_insert('pp_taxonomy_manager_terms')
      ->fields(array(
        'tid' => $term->tid,
        'vid' => $term->vid,
        'tmid' => $this->config->getId(),
        'synced' => $start_time,
        'uri' => $uri,
        'hash' => $hash,
      ))
      ->execute();
  }

  /**
   * Updates the hash data for a taxonomy term.
   *
   * @param object $term
   *   The taxonomy term.
   * @param string $hash
   *   The new hash data.
   * @param int $start_time
   *   The synchonization start time.
   */
  protected function updateHashData($term, $hash, $start_time) {
    db_update('pp_taxonomy_manager_terms')
      ->fields(array(
        'synced' => $start_time,
        'hash' => $hash,
      ))
      ->condition('vid', $term->vid)
      ->condition('tid', $term->tid)
      ->execute();
  }

  /**
   * Creates a hash code from a concept $concept.
   *
   * @param object $concept
   *   A concept object from PoolParty.
   *
   * @return string
   *   The hash code.
   */
  protected function hash($concept) {
    return hash('md5', serialize($concept));
  }

  /**
   * Returns all SKOS properties of the taxonomy fields.
   *
   * @return array
   *   List of SKOS properties.
   */
  protected function skosProperties() {
    $fields = self::taxonomyFields();
    $properties = array();
    foreach ($fields as $field) {
      if (isset($field->property)) {
        $properties[] = $field->property;
      }
    }
    
    $properties[] = 'skos:broader';

    return $properties;
  }

  /**
   * Creates a machine readable name from a human readable name.
   *
   * @param string $name
   *   The human readable name.
   *
   * @return string
   *   The machine readable name.
   */
  public static function machineName($name) {
    $name = strtolower($name);
    return preg_replace(array('@[^a-z0-9_]+@', '@_+@'), '_', $name);
  }

  /**
   * Returns the URI with language of a concept.
   *
   * @param object $concept
   *   The object of a concept.
   *
   * @return string
   *   The uri with the language (e.g., http://a.concept.uri/1234@en).
   */
  protected function getUri($concept) {
    return $concept->uri . '@' . $concept->ppLang;
  }

}
